@@ -8,8 +8,15 @@ import copy
 import re
 import uuid
 
-from .constants import PREFORMATTED_TAGS, IMPLICIT_SELF_CLOSING_TAGS, TAG_NAMES_TO_ADDITIONAL_ATTRIBUTES, COMMON_JAVASCRIPT_ATTRIBUTES, ALL_JAVASCRIPT_EVENT_ATTRIBUTES, TAG_ITEM_BINARY_ATTRIBUTES, TAG_ITEM_ATTRIBUTE_LINKS, TAG_ITEM_ATTRIBUTES_SPECIAL_VALUES, TAG_ITEM_CHANGE_NAME_FROM_ATTR, TAG_ITEM_CHANGE_NAME_FROM_ITEM
-from .SpecialAttributes import SpecialAttributesDict, StyleAttribute, AttributeNodeMap
+from .conversions import convertBooleanStringToBoolean
+
+from .constants import ( PREFORMATTED_TAGS, IMPLICIT_SELF_CLOSING_TAGS, TAG_NAMES_TO_ADDITIONAL_ATTRIBUTES,
+    COMMON_JAVASCRIPT_ATTRIBUTES, ALL_JAVASCRIPT_EVENT_ATTRIBUTES, TAG_ITEM_BINARY_ATTRIBUTES,
+    TAG_ITEM_ATTRIBUTE_LINKS, TAG_ITEM_ATTRIBUTES_SPECIAL_VALUES, TAG_ITEM_CHANGE_NAME_FROM_ATTR,
+    TAG_ITEM_CHANGE_NAME_FROM_ITEM, TAG_ITEM_BINARY_ATTRIBUTES_STRING_ATTR,
+)
+
+from .SpecialAttributes import SpecialAttributesDict, StyleAttribute, AttributeNodeMap, DOMTokenList
 
 from .utils import escapeQuotes, tostr, stripWordsOnly
 
@@ -185,6 +192,13 @@ class AdvancedTag(object):
             if name in TAG_ITEM_CHANGE_NAME_FROM_ITEM: 
                 name = TAG_ITEM_CHANGE_NAME_FROM_ITEM[name]
 
+            if name in TAG_ITEM_BINARY_ATTRIBUTES_STRING_ATTR:
+                # NOTE: These are binary attributes, but have a string representation within html (e.x. spellcheck)
+                #         Handle the value conversion etc. within the SpecialAttributesDict
+                self.setAttribute(name, value)
+
+                # Return the converted attribute
+                return self.getAttribute(name)
             # Check if the value needs to be converted to a binary/boolean, e.x. "checked"
             elif name in TAG_ITEM_BINARY_ATTRIBUTES:
                 # If it is a boolean html attribute, it is denoted by the presence of the field at all (any value means "True"/set/yes)
@@ -256,9 +270,14 @@ class AdvancedTag(object):
                 return TAG_ITEM_ATTRIBUTES_SPECIAL_VALUES[name](self)
 
             # Check if given attribute has a different name via dot-access and actual attribute ( e.x. "className" -> "class" )
+            #   TODO
             if name in TAG_ITEM_CHANGE_NAME_FROM_ITEM:
                 name = TAG_ITEM_CHANGE_NAME_FROM_ITEM[name]
 
+            # These have a binary representation in dot-access (this method), but a string value as an attribute.
+            if name in TAG_ITEM_BINARY_ATTRIBUTES_STRING_ATTR:
+                return convertBooleanStringToBoolean( self.getAttribute(name) )
+
             # Check if this is a binary/boolean attribute, i.e. the value is always either True or False ( e.x. "checked" )
             elif name in TAG_ITEM_BINARY_ATTRIBUTES:
                 val = self.getAttribute(name, False)
@@ -543,7 +562,7 @@ class AdvancedTag(object):
 
     def appendBlock(self, block):
         '''
-            appendBlock - Append a block to this element. A block can be a string (text node), or an AdvancedTag (tag node)
+            append / appendBlock - Append a block to this element. A block can be a string (text node), or an AdvancedTag (tag node)
 
             @param <str/AdvancedTag> - block to add
 
@@ -560,6 +579,9 @@ class AdvancedTag(object):
         
         return block
 
+    # append - Alias (official API name) for appendBlock
+    append = appendBlock
+
 
     def appendBlocks(self, blocks):
         '''
@@ -1050,6 +1072,53 @@ class AdvancedTag(object):
 
         return [block for block in myBlocks if not issubclass(block.__class__, AdvancedTag)]
 
+    @property
+    def innerText(self):
+        '''
+            innerText - property, gets the text of just this node. Use #textContent for this node and all children
+
+                    This is an alias of the .text property
+
+                 @return <str> - The text of this node
+        '''
+        return self.text
+
+
+    @property
+    def textContent(self):
+        '''
+            textContent - property, gets the text of this node and all inner nodes.
+
+                Use .innerText for just this node's text
+
+              @return <str> - The text of all nodes at this level or lower
+        '''
+
+        def _collateText(curNode):
+            '''
+                _collateText - Recursive function to gather the "text" of all blocks
+
+                                 in the order that they appear
+
+                    @param curNode <AdvancedTag> - The current AdvancedTag to process
+
+                    @return list<str> - A list of strings in order. Join using '' to obtain text
+                                            as it would appear
+            '''
+                   
+            curStrLst = []
+            blocks = object.__getattribute__(curNode, 'blocks')
+
+            for block in blocks:
+                if isTagNode(block):
+                    curStrLst += _collateText(block)
+                else:
+                    curStrLst.append(block)
+
+            return curStrLst
+
+        return ''.join(_collateText(self))
+
 
     def getBlocksText(self):
         '''
@@ -1212,6 +1281,7 @@ class AdvancedTag(object):
         # Otherwise, get all children of parent excluding this node
         return TagCollection([peer for peer in peers if peer is not self])
 
+
     @property
     def peers(self):
         '''
@@ -1282,16 +1352,16 @@ class AdvancedTag(object):
 
             @return <str> - Class attribute, or empty string if not set
         '''
-        return ' '.join(self.classList)
+        return str(self.classList)
 
     @property
     def classList(self):
         '''
             classList - get a copy of the list of the class names ( the "class" attribute ) for this element
 
-                @return list<str> - A list of the class names for this element
+                @return DOMTokenList<str> - A list of the class names for this element
         '''
-        return self._classNames[:]
+        return DOMTokenList(self._classNames[:])
 
     def getUid(self):
         '''
@@ -1328,7 +1398,8 @@ class AdvancedTag(object):
             # Get all attributes
             val = tostr(val)
 
-            if val:
+            # Only binary attributes have a "present/not present"
+            if val or name not in TAG_ITEM_BINARY_ATTRIBUTES:
                 # Escape any quotes found in the value
                 val = escapeQuotes(val)
 
@@ -1806,6 +1877,45 @@ class AdvancedTag(object):
 
         return None
 
+    def getParentElementCustomFilter(self, filterFunc):
+        '''
+            getParentElementCustomFilter - Runs through parent on up to document root, returning the
+
+                                              first tag which filterFunc(tag) returns True.
+
+                @param filterFunc <function/lambda> - A function or lambda expression that should return "True" if the passed node matches criteria.
+
+                @return <AdvancedTag/None> - First match, or None
+
+
+                @see getFirstElementCustomFilter for matches against children
+        '''
+        parentNode = self.parentNode
+        while parentNode:
+
+            if filterFunc(parentNode) is True:
+                return parentNode
+
+            parentNode = parentNode.parentNode
+
+        return None
+
+
+    def getPeersCustomFilter(self, filterFunc):
+        '''
+            getPeersCustomFilter - Get elements who share a parent with this element and also pass a custom filter check
+
+                @param filterFunc <lambda/function> - Passed in an element, and returns True if it should be treated as a match, otherwise False.
+
+                @return <TagCollection> - Resulting peers, or None if no parent node.
+        '''
+        peers = self.peers
+        if peers is None:
+            return None
+
+        return TagCollection([peer for peer in peers if filterFunc(peer) is True])
+
+
     def getPeersByAttr(self, attrName, attrValue):
         '''
             getPeersByAttr - Gets peers (elements on same level) which match an attribute/value combination.
